using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RssToolkit;
using BlogML.Xml;
using System.IO;
using System.Xml;

namespace Rss2BlogML
{
	public partial class mainForm : Form
	{

		public mainForm()
		{
			InitializeComponent();
		}

		private void button1_Click(object sender, EventArgs e)
		{
			ToggleButtons(false);

			if(dataGridView1.Rows.Count==1)
			{
				ShowMessage("No feed specified");
				return;
			}

			int mainCount = 0;
			foreach (DataGridViewRow row in dataGridView1.Rows)
			{
				if (!row.IsNewRow)
				{
					bool isMain = Convert.ToBoolean(row.Cells["IsMain"].Value);
					if (isMain)
						mainCount++;
				}
			}

			if (mainCount == 0)
			{
				ShowMessage("At least one feed must be marked as main feed");
				return;
			}
			if (mainCount > 1)
			{
				ShowMessage("There cannot be more than one feed marked as main feed");
				return;
			}

			if (saveFileDialog1.ShowDialog() == DialogResult.OK)
			{
				Cursor = Cursors.WaitCursor;
				progressBar.Value = 0;
				backgroundWorker1.RunWorkerAsync((string)cmbRssDialect.SelectedValue);
			}
		}

		private void ShowMessage(string message)
		{
			MessageBox.Show(message,"Rss2BlogML");
			ToggleButtons(true);
		}

		private void ToggleButtons(bool enable)
		{
			btnConvert.Enabled = enable;
			btnCancel.Enabled = !enable;
		}

		private void mainForm_Load(object sender, EventArgs e)
		{
			stringDictionaryBindingSource.DataSource = RssMapperFactory.Current.GeneratorList;
			cmbRssDialect.DisplayMember = "Value";
			cmbRssDialect.ValueMember = "Key";
		}

		private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
		{
			BackgroundWorker wk = ((BackgroundWorker)sender);
			string fileName = saveFileDialog1.FileName;

			BlogMLBlog blog = new BlogMLBlog();

			int i = 0;
			int totRows = dataGridView1.Rows.Count - 1;
			foreach (DataGridViewRow row in dataGridView1.Rows)
			{
				if (!row.IsNewRow)
				{
					if (wk.CancellationPending)
						e.Cancel = true;
					else
					{
						string category = string.Empty;
						if (row.Cells["Category"].Value != null)
							category = row.Cells["Category"].Value.ToString();
						bool isMain = Convert.ToBoolean(row.Cells["IsMain"].Value);
						RssGrabber grabber = new RssGrabber(RssMapperFactory.Current.Create((string)e.Argument,category,blog), blog);
						grabber.Grab(row.Cells["RssUrl"].Value.ToString(), isMain);
						wk.ReportProgress(++i * 100 / totRows);
					}
				}
			}

			if (blog.Posts.Count > 0)
			{
				BlogMLWriter writer = new BlogMLWriter(blog);

				using (XmlTextWriter xmlWriter = new XmlTextWriter(fileName, Encoding.UTF8))
				{
					xmlWriter.Formatting = Formatting.Indented;
					writer.Write(xmlWriter);
					xmlWriter.Flush();
				}
				e.Result="BlogML export saved to " + fileName + Environment.NewLine + "Number of post exported: " + blog.Posts.Count;
			}
			else
			{
				e.Result="No post found at the urls specified";
			}
		}

		private void backgroundWorker1_ProgressChanged(object sender, ProgressChangedEventArgs e)
		{
			progressBar.Value = e.ProgressPercentage;
		}

		private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			if (e.Error != null)
				ShowMessage(e.Error.Message);
			else if (e.Cancelled)
				ShowMessage("Operation cancelled by the user");
			else
				ShowMessage((string)e.Result);
			Cursor = Cursors.Default;
		}

		private void btnCancel_Click(object sender, EventArgs e)
		{
			backgroundWorker1.CancelAsync();
			btnCancel.Enabled = false;
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			backgroundWorker1.CancelAsync();
			backgroundWorker1.Dispose();
			Application.Exit();
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			new about().ShowDialog();
		}



	}
}