'Community Server 2007 BlogML converter Beta
'Copyright Keyvan Nayyeri (www.nayyeri.net) - 2007
'BlogML Writer and Reader classes are provided by BlogML project (www.blogml.com)
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.Specialized
Imports System.IO
Imports System.Web
Imports BlogML
Imports BlogML.Xml
Imports CommunityServer
Imports CommunityServer.Blogs.Components
Imports CommunityServer.Components

Namespace CommunityServer.BlogML
    Public Class Reader
#Region "Private members"
        Private _Blog As Weblog
        Private _SitePath As String
        Private _BlogCommentCount As Integer = 0
        Private _BlogTrackBackCount As Integer = 0
#End Region
#Region "Constructor"
        Sub New(ByVal ApplicationKey As String)
            Me._Blog = Weblogs.GetWeblog(ApplicationKey)
        End Sub
#End Region
#Region "Main process"
        Public Sub LoadBlog(ByVal XML As String, ByVal ClearOldItems As Boolean)
            If ClearOldItems = True Then
                Clear()
            End If

            Dim Blog As New BlogMLBlog
            Try
                Blog = BlogMLSerializer.Deserialize(New StringReader(XML))
            Catch ex As Exception
                Throw New Exception("Couldn't load your BlogML file. Maybe it is wellformedness")
            End Try

            LoadBlogProperties(Blog)

            Dim Categories As Hashtable
            Categories = ReadCategories(Blog)

            LoadPosts(Blog, Categories)

            UpdateStats(Blog)
        End Sub
#End Region
#Region "Private methods"
        Private Sub Clear()
            ClearCategories()
            ClearPosts()
        End Sub
        Private Sub ClearCategories()
            Dim Categories As List(Of PostCategory) = PostCategories.GetCategories(Me._Blog.SectionID, True)

            For Each Category As PostCategory In Categories
                Category.IsEnabled = True
                PostCategories.DeleteCategory(Category.CategoryID, Category.SectionID)
            Next
        End Sub
        Private Sub ClearPosts()
            Dim Threads As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._Blog.PostCount + Me._Blog.ArticleCount
            Query.BlogPostType = BlogPostType.Post Or BlogPostType.Article

            Threads = WeblogPosts.GetBlogThreads(Query)

            For Each Thread As Post In Threads.Threads
                Dim Post As New WeblogPost
                Post = WeblogPosts.GetPost(Thread.PostID, True, Thread.IsApproved, False)

                ClearPostComments(Post)
                ClearPostTrackBacks(Post)

                WeblogPosts.Delete(Post, CSContext.Current.UserID)
            Next

        End Sub
        Private Sub ClearPostComments(ByVal Post As WeblogPost)
            Dim Threads As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._Blog.CommentCount
            Query.BlogPostType = BlogPostType.Comment
            Query.ParentID = Post.PostID

            Threads = Blogs.Components.WeblogPosts.GetBlogThreads(Query)

            For Each Thread As Post In Threads.Threads
                Dim Comment As WeblogPost = WeblogPosts.GetPost(Thread.PostID, True, Thread.IsApproved, False)
                WeblogPosts.Delete(Comment, CSContext.Current.UserID)
            Next
        End Sub
        Private Sub ClearPostTrackBacks(ByVal Post As WeblogPost)
            Dim Threads As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._Blog.TrackbackCount
            Query.BlogPostType = BlogPostType.Trackback
            Query.ParentID = Post.PostID

            Threads = Blogs.Components.WeblogPosts.GetBlogThreads(Query)

            For Each Thread As Post In Threads.Threads
                Dim TrackBack As WeblogPost = WeblogPosts.GetPost(Thread.PostID, True, Thread.IsApproved, False)
                WeblogPosts.Delete(TrackBack, CSContext.Current.UserID)
            Next
        End Sub
        Private Sub LoadBlogProperties(ByVal Blog As BlogMLBlog)
            Dim Properties As BlogMLBlog.ExtendedPropertiesCollection = Blog.ExtendedProperties

            For Each ExtendedProperty As Pair(Of String, String) In Properties
                CheckAndGetProperty(ExtendedProperty)
            Next
        End Sub
        Private Sub CheckAndGetProperty(ByVal ExtendedProperty As Pair(Of String, String))
            Select Case ExtendedProperty.Key
                Case BlogMLBlogExtendedProperties.CommentModeration
                    Me._Blog.ModerationTypeDefault = GetCommentModerationType(ExtendedProperty.Value)
                Case BlogMLBlogExtendedProperties.EnableSendingTrackbacks
                    Me._Blog.EnableTrackbacksDefault = GetSendTrackbackType(ExtendedProperty.Value)
            End Select
            Weblogs.Update(Me._Blog)
        End Sub
        Private Function GetCommentModerationType(ByVal strModerationType As String) As CommentModerationType
            Dim ModerationType As CommentModerationTypes = CType(strModerationType, CommentModerationTypes)

            Select Case ModerationType
                Case CommentModerationTypes.Disabled
                    Return CommentModerationType.Ignore
                Case CommentModerationTypes.Enabled
                    Return CommentModerationType.All
                Case CommentModerationTypes.Anonymous
                    Return CommentModerationType.Anonymous
                Case Else
                    Return CommentModerationType.None
            End Select
        End Function
        Private Function GetSendTrackbackType(ByVal strSendTrackbackType As String) As Boolean
            Dim SendTrackbackType As SendTrackbackTypes = CType(strSendTrackbackType, SendTrackbackTypes)

            Select Case SendTrackbackType
                Case SendTrackbackTypes.No
                    Return False
                Case SendTrackbackTypes.Yes
                    Return True
            End Select
        End Function
        Private Function ReadCategories(ByVal Blog As BlogMLBlog) As Hashtable
            Dim CategoriesHash As New Hashtable
            For Each Category As BlogMLCategory In Blog.Categories
                Dim NewCategory As New PostCategory
                With NewCategory
                    .DateCreated = Category.DateCreated
                    .Description = Category.Description
                    .IsEnabled = Category.Approved
                    .Name = Category.Title
                    .ParentID = 0
                    .SectionID = Me._Blog.SectionID

                    Try
                        PostCategories.CreateCategory(NewCategory)
                    Catch ex As Exception
                        Throw New ArgumentException("Error when tried to add categories to database")
                    End Try
                    CategoriesHash.Add(Category.ID, Category.Title)
                End With
            Next
            Return CategoriesHash
        End Function
        Private Sub LoadPosts(ByVal Blog As BlogMLBlog, ByVal CategoryHash As Hashtable)
            Dim LastPostAuthor As String = Nothing
            Dim LastPostAuthorID As Integer = 0
            Dim LastPostDate As Date = Nothing
            Dim LastPostName As String = Nothing
            Dim LastPostSubject As String = Nothing


            For Each Post As BlogMLPost In Blog.Posts
                Dim NewPost As New WeblogPost

                With NewPost
                    .Body = Post.Content.UncodedText
                    If Post.PostType = BlogPostTypes.Normal Then
                        .BlogPostType = BlogPostType.Post
                    ElseIf Post.PostType = BlogPostTypes.Article Then
                        .BlogPostType = BlogPostType.Article
                    End If
                    .Subject = Post.Title
                    .ThreadDate = Post.DateCreated
                    .UserTime = Post.DateCreated
                    .PostDate = Post.DateCreated
                    .UserTime = Post.DateCreated
                    .SectionID = Me._Blog.SectionID
                    .Views = Post.Views
                    '.Name = Post.PostName
                    .SetExtendedAttribute("EverPublished", CType(Post.Approved, Boolean))
                    .IndexInThread = True
                    .IsApproved = True
                    .IsAggregated = True
                    .IsLocked = False
                    .IsAggregated = True
                    .IsCommunityAggregated = True
                    .PostLevel = 1
                    .PostType = PostContentType.HTML
                End With
                'Add categories
                If Post.Categories.Count > 0 Then
                    Dim NewPostCats(Post.Categories.Count) As String
                    For i As Integer = 0 To Post.Categories.Count - 1
                        NewPostCats(i) = CategoryHash(Post.Categories(i).Ref)
                    Next
                    NewPost.Categories = NewPostCats
                End If

                WeblogPosts.Add(NewPost)

                'Temporary saving stats
                LastPostAuthor = NewPost.Username
                LastPostAuthorID = NewPost.AuthorID
                LastPostDate = NewPost.PostDate
                LastPostName = NewPost.Name
                LastPostSubject = NewPost.Subject

                LoadComments(NewPost, Post)
                LoadTrackBacks(NewPost, Post)
                LoadAttachments(NewPost, Post)
            Next

            'Update blog stats
            Me._Blog.MostRecentPostAuthor = LastPostAuthor
            Me._Blog.MostRecentPostAuthorID = LastPostAuthorID
            Me._Blog.MostRecentPostDate = LastPostDate
            Me._Blog.MostRecentPostName = LastPostName
            Me._Blog.MostRecentPostSubject = LastPostSubject
        End Sub
        Private Sub LoadComments(ByVal NewPost As Post, ByVal Post As BlogMLPost)
            If Post.Comments.Count > 0 Then
                For Each Comment As BlogMLComment In Post.Comments
                    Dim NewComment As New WeblogPost

                    With NewComment
                        .Body = Comment.Content.UncodedText
                        .BlogPostType = BlogPostType.Comment
                        .Subject = Comment.Title
                        .ThreadDate = Comment.DateCreated
                        .UserTime = Comment.DateCreated
                        .PostDate = Comment.DateCreated
                        .UserTime = Comment.DateCreated
                        .SectionID = Me._Blog.SectionID
                        .SetExtendedAttribute("EverPublished", True)
                        .SetExtendedAttribute("TitleUrl", Comment.UserUrl)
                        .SetExtendedAttribute("SubmittedUserName", Comment.UserName)
                        .IsApproved = CType(Comment.Approved, Boolean)
                        .ParentID = NewPost.PostID
                        .ThreadID = NewPost.ThreadID
                        .PostLevel = 2
                        .PostType = PostContentType.HTML
                    End With


                    Dim User As New User
                    User.UserID = 2101
                    User.Username = "Anonymous"

                    WeblogPosts.Add(NewComment, User)

                    'Update blog counter
                    Me._BlogCommentCount += 1
                Next
            End If
        End Sub
        Private Sub LoadTrackBacks(ByVal NewPost As Post, ByVal Post As BlogMLPost)
            If Post.Trackbacks.Count > 0 Then
                For Each TrackBack As BlogMLTrackback In Post.Trackbacks
                    Dim NewTrackBack As New WeblogPost

                    With NewTrackBack
                        .Body = TrackBack.Title
                        .BlogPostType = BlogPostType.Trackback
                        .Subject = TrackBack.Title
                        .ThreadDate = TrackBack.DateCreated
                        .UserTime = TrackBack.DateCreated
                        .PostDate = TrackBack.DateCreated
                        .UserTime = TrackBack.DateCreated
                        .SectionID = Me._Blog.SectionID
                        .SetExtendedAttribute("EverPublished", True)
                        .SetExtendedAttribute("TitleUrl", TrackBack.Url)
                        .SetExtendedAttribute("trackbackName", "TrackBack")
                        .IsApproved = CType(TrackBack.Approved, Boolean)
                        .ParentID = NewPost.PostID
                        .ThreadID = NewPost.ThreadID
                        .PostLevel = 2
                        .PostType = PostContentType.Text
                    End With



                    Dim User As New User
                    User.UserID = 2101
                    User.Username = "Anonymous"

                    WeblogPosts.Add(NewTrackBack, User)


                    'Update blog counter
                    Me._BlogTrackBackCount += 1
                Next
            End If
        End Sub
        Private Sub LoadAttachments(ByVal NewPost As Post, ByVal Post As BlogMLPost)
            If Post.Attachments.Count > 0 Then
                Dim objCDP As CommonDataProvider = CommonDataProvider.Instance
                Dim Attachment As New PostAttachment

                If Post.Attachments(0).Embedded = False Then
                    'Remote attachment
                    Attachment.FileName = Post.Attachments(0).Url
                    Attachment.Content = Nothing

                    Dim TempData(0) As Byte
                    Attachment.Content = GetStream(TempData)
                    Attachment.IsRemote = True
                    Attachment.ContentType = Post.Attachments(0).MimeType
                Else
                    'Local attachment
                    Attachment.FileName = Path.GetFileName(Post.Attachments.Item(0).Url)
                    Attachment.Content = GetStream(Post.Attachments.Item(0).Data)
                    Attachment.IsRemote = False
                    Attachment.ContentType = Post.Attachments(0).MimeType
                End If

                objCDP.AddPostAttachment(NewPost, Attachment)
            End If
        End Sub
        Private Sub UpdateStats(ByVal Blog As BlogMLBlog)
            Me._Blog.PostCount += Blog.Posts.Count
            Me._Blog.CommentCount += Me._BlogCommentCount
            Me._Blog.TrackbackCount += Me._BlogTrackBackCount
        End Sub
        Private Function GetStream(ByVal Content As Byte()) As Stream
            Dim Output As New MemoryStream
            Output.Read(Content, 0, Content.Length)
            Return Output
        End Function
#End Region
    End Class
End Namespace