'Community Server 2007 BlogML converter Beta
'Copyright Keyvan Nayyeri (www.nayyeri.net) - 2007
'BlogML Writer and Reader classes are provided by BlogML project (www.blogml.com)
Imports System
Imports System.IO
Imports BlogML
Imports BlogML.Xml
Imports CommunityServer
Imports CommunityServer.Blogs.Components
Imports CommunityServer.Components
Namespace CommunityServer.BlogML
    Public Class Writer
        Inherits BlogMLWriterBase
#Region "Private Members"
        Private _Blog As Blogs.Components.Weblog
        Private _PostPageSize As Integer
        Private _CommentPageSize As Integer
        Private _TrackBackPageSize As Integer
#End Region
#Region "Constructor"
        Sub New(ByVal ApplicationKey As String)
            Me._Blog = Weblogs.GetWeblog(ApplicationKey)

            Me._PostPageSize = 10 + 2 * (Me._Blog.PostCount + Me._Blog.ArticleCount)
            Me._CommentPageSize = 10 + 2 * (Me._Blog.CommentCount)
            Me._TrackBackPageSize = 10 + 2 * Me._Blog.TrackbackCount
        End Sub
#End Region
#Region "Main process"
        Protected Overrides Sub InternalWriteBlog()
            WriteBlogInfo() 'Start
            WriteCategories()
            WritePosts()

            MyBase.WriteEndElement() 'Blog
        End Sub
#End Region
#Region "Private methods"
        Private Sub WriteBlogInfo()
            MyBase.WriteStartBlog(Me._Blog.Name, Me._Blog.Description, Me._Blog.Url)
            WriteBlogAuthors()
            WriteBlogExtendedProperties()
        End Sub
        Private Sub WriteBlogAuthors()
            MyBase.WriteStartAuthors()
            If Me._Blog.OwnerArray.Length > 0 Then
                For Each Author As String In Me._Blog.OwnerArray()
                    Dim objUser As User = Users.FindUserByUsername(Author)
                    MyBase.WriteAuthor(objUser.UserID, objUser.Username, objUser.Email, objUser.DateCreated, _
                    objUser.LastLogin, Not objUser.IsBanned)
                Next
            Else
                Throw New Exception("Blog must have at least one owner!")
            End If
            MyBase.WriteEndElement() 'End Authors
        End Sub
        Private Sub WriteBlogExtendedProperties()
            MyBase.WriteStartExtendedProperties()

            MyBase.WriteExtendedProperty(BlogMLBlogExtendedProperties.CommentModeration, _
            GetCommentModerationProperty(Me._Blog.ModerationTypeDefault).ToString())

            MyBase.WriteExtendedProperty(BlogMLBlogExtendedProperties.EnableSendingTrackbacks, _
            GetSendTrackbackProperty(Me._Blog.EnableTrackbacksDefault).ToString())

            MyBase.WriteEndElement()
        End Sub
        Private Function GetCommentModerationProperty(ByVal ModerationType As CommentModerationType) As CommentModerationTypes
            Select Case ModerationType
                Case CommentModerationType.None
                    Return CommentModerationTypes.Disabled
                Case CommentModerationType.All
                    Return CommentModerationTypes.Enabled
                Case CommentModerationType.Anonymous
                    Return CommentModerationTypes.Anonymous
                Case Else
                    Return CommentModerationTypes.Disabled
            End Select
        End Function
        Private Function GetSendTrackbackProperty(ByVal Send As Boolean) As SendTrackbackTypes
            Select Case Send
                Case False
                    Return SendTrackbackTypes.No
                Case True
                    Return SendTrackbackTypes.Yes
                Case Else
                    Return SendTrackbackTypes.Yes
            End Select
        End Function
        Private Sub WriteCategories()
            Dim Categories As List(Of PostCategory) = PostCategories.GetCategories(Me._Blog.SectionID, True)
            If Categories.Count <> 0 Then
                MyBase.WriteStartCategories()
                For Each Category As PostCategory In Categories
                    With Category
                        MyBase.WriteCategory(.CategoryID, .Name, .DateCreated, .DateCreated, _
                        .IsEnabled, .Description, .ParentID)
                    End With
                Next
                MyBase.WriteEndElement() 'End Categories
            End If
        End Sub
        Private Sub WritePosts()
            Dim Threads As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._PostPageSize
            Query.BlogPostType = BlogPostType.Post Or BlogPostType.Article


            Threads = WeblogPosts.GetBlogThreads(Query)

            If Threads.Threads.Count <> 0 Then
                MyBase.WriteStartPosts()

                For Each objPost As Post In Threads.Threads
                    Dim Post As WeblogPost = WeblogPosts.GetPost(objPost.PostID, True, objPost.IsApproved, False)

                    With Post
                        Dim objPostType As BlogPostTypes
                        If Post.BlogPostType = BlogPostType.Post Then
                            objPostType = BlogPostTypes.Normal
                        ElseIf Post.BlogPostType = BlogPostType.Article Then
                            objPostType = BlogPostTypes.Article
                        End If

                        Dim strPostUrl As String = GetPostUrl(Post)
                        If objPost.HasExcerpt = False Then
                            MyBase.WriteStartPost(.PostID, .Subject, .PostDate, .PostDate, .IsApproved, .Body, _
                            strPostUrl, .Views, objPostType, .Name)
                        Else
                            MyBase.WriteStartPost(.PostID, .Subject, .PostDate, .PostDate, .IsApproved, .Body, _
                            strPostUrl, .Views, objPost.Excerpt, objPostType, .Name)
                        End If
                    End With
                    WritePostCategories(Post)
                    WritePostComments(Post)
                    WritePostTrackBacks(Post)
                    WritePostAttachments(Post)
                    WritePostAuthors(Post)

                    MyBase.WriteEndElement() 'End Post
                Next
                MyBase.WriteEndElement() 'End Posts
            End If
        End Sub
        Private Sub WritePostCategories(ByVal Post As WeblogPost)
            If Post.HasCategories = True Then
                Dim Categories() As String = Post.Categories
                MyBase.WriteStartCategories()

                For Each Cat As String In Categories
                    Dim Category As PostCategory = PostCategories.GetCategory(Cat, Me._Blog.SectionID)
                    MyBase.WriteCategoryReference(Category.CategoryID)
                Next
                MyBase.WriteEndElement() 'End Categories
            End If
        End Sub
        Private Sub WritePostComments(ByVal Post As WeblogPost)
            Dim Comments As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._CommentPageSize
            Query.BlogPostType = BlogPostType.Comment
            Query.ParentID = Post.PostID

            Comments = WeblogPosts.GetBlogThreads(Query, False)

            If Comments.Threads.Count <> 0 Then
                MyBase.WriteStartComments()
                For Each Comment As Post In Comments.Threads
                    With Comment
                        Dim strCommenter As String = .GetExtendedAttribute("SubmittedUserName")
                        Dim strUrl As String = .GetExtendedAttribute("TitleUrl")

                        If String.IsNullOrEmpty(strCommenter) Then strCommenter = .Username
                        If String.IsNullOrEmpty(strCommenter) Then strCommenter = "anonymous"
                        If String.IsNullOrEmpty(strUrl) Then strCommenter = "http://"

                        MyBase.WriteComment(.PostID, .Subject, .PostDate, .PostDate, .IsApproved, strCommenter _
                        , Nothing, strUrl, .Body)
                    End With
                Next
                Me.WriteEndElement() 'End Comments
            End If
        End Sub
        Private Sub WritePostTrackBacks(ByVal Post As WeblogPost)
            Dim Trackbacks As New ThreadSet
            Dim Query As New BlogThreadQuery
            Query.SectionID = Me._Blog.SectionID
            Query.PageSize = Me._TrackBackPageSize
            Query.BlogPostType = BlogPostType.Trackback
            Query.ParentID = Post.PostID

            Trackbacks = WeblogPosts.GetBlogThreads(Query, False)

            If Trackbacks.Threads.Count <> 0 Then
                MyBase.WriteStartTrackbacks()

                For Each Trackback As Post In Trackbacks.Threads
                    With Trackback
                        Dim strName = .Subject
                        Dim strUrl As String = .GetExtendedAttribute("TitleUrl")

                        If String.IsNullOrEmpty(strUrl) = True Then strUrl = .GetExtendedAttribute("TitleUrl")
                        If String.IsNullOrEmpty(strUrl) = True Then strUrl = "http://"

                        MyBase.WriteTrackback(.PostID, strName, .PostDate, .PostDate, .IsApproved, strUrl)
                    End With
                Next
                MyBase.WriteEndElement() 'End TrackBackss
            End If
        End Sub
        Private Sub WritePostAttachments(ByVal Post As WeblogPost)
            If Post.HasAttachment = True Then
                MyBase.WriteStartAttachments()
                If Post.Attachment.IsRemote = True Then
                    Dim strMIME As String = "none"
                    If (Not String.IsNullOrEmpty(Post.Attachment.ContentType)) Then
                        strMIME = Post.Attachment.ContentType
                    End If
                    MyBase.WriteAttachment(Post.Attachment.FileName, strMIME, Post.Attachment.FileName)
                Else
                    Dim objCDP As CommonDataProvider = CommonDataProvider.Instance
                    Dim Attachment As New PostAttachment
                    Attachment = objCDP.GetPostAttachment(Post.PostID, False)

                    MyBase.WriteAttachment(Attachment.FileName, Attachment.ContentType, Attachment.Content)
                End If
                MyBase.WriteEndElement() 'End Attachments
            End If
        End Sub
        Private Sub WritePostAuthors(ByVal Post As WeblogPost)
            MyBase.WriteStartAuthors()
            MyBase.WriteAuthorReference(Post.AuthorID)
            MyBase.WriteEndElement()
        End Sub
        Private Function GetPostUrl(ByVal Post As WeblogPost) As String
            Return Post.ViewPostURL
        End Function
#End Region
    End Class
End Namespace



