using System;
using System.Collections.Generic;
using System.Text;
using BlogML;
using BlogML.Xml;
using System.Collections.Specialized;

namespace Rss2BlogML
{
	public class DotTextRssMapper : IRssMapper
	{

		private static int lastAuthorId = 1;
		private static int lastCategoryId = 1;

		#region IRssMapper Members

		private string _category;
		private BlogMLBlog _blog;

		public DotTextRssMapper(string category, BlogMLBlog blog)
		{
			_category = category;
			_blog = blog;
		}

		public virtual string GeneratePostId(RssToolkit.GenericRssElement rssElement)
		{
			string commentRssUrl = Parsers.ParseString(rssElement.Attributes, "wfw:commentRss");
			Uri commentRssUri = new Uri(commentRssUrl);
			string filename = commentRssUri.Segments[commentRssUri.Segments.Length-1];

			return filename.Substring(0,filename.Length-5);
		}

		public string GenerateCommentId(RssToolkit.GenericRssElement rssElement)
		{
			string commentUrl = Parsers.ParseString(rssElement.Attributes, "guid");
			Uri commentUri = new Uri(commentUrl);
			return commentUri.Fragment.Substring(1);
		}

		public IdStruct GenerateAuthorId(RssToolkit.GenericRssElement rssElement)
		{
			string authorname = Parsers.ParseString(rssElement.Attributes, "dc:creator");
			foreach (BlogMLAuthor author in _blog.Authors)
			{
				if (author.Title.Equals(authorname))
					return new IdStruct(author.ID,false);
			}
			return new IdStruct(String.Format("{0:d}", lastAuthorId++),true);
		}

		public CatStruct[] GenerateCategoryIds(RssToolkit.GenericRssElement rssElement)
		{
			StringCollection categoryList = new StringCollection();
			string categoryInFeed = Parsers.ParseString(rssElement.Attributes, "category");


			string[] categories = categoryInFeed.Split(',');
			foreach (string cat in categories)
			{
				string trimmedCat = cat.Trim();
				if (!String.IsNullOrEmpty(trimmedCat))
					if (!trimmedCat.Equals(_category))
						categoryList.Add(trimmedCat);
			}
			if (!String.IsNullOrEmpty(_category))
				categoryList.Add(_category);

			CatStruct[] returnArray = new CatStruct[categoryList.Count];

			int i = 0;
			foreach (string catName in categoryList)
			{
				bool foundCat = false;
				foreach (BlogMLCategory category in _blog.Categories)
				{
					if (category.Title.Equals(catName))
					{
						returnArray[i] = new CatStruct(catName, category.ID, false);
						foundCat = true;
					}
				}
				if(!foundCat)
					returnArray[i] = new CatStruct(catName, String.Format("{0:d}", lastCategoryId++), true);
				i++;
			}

			return returnArray;
		}

		public string GenerateTrackbackId(RssToolkit.GenericRssElement rssElement)
		{
			throw new Exception("The method or operation is not implemented.");
		}


		public string GeneratePostName(RssToolkit.GenericRssElement rssElement)
		{
			string url = Parsers.ParseString(rssElement.Attributes, "guid");
			Uri uri = new Uri(url);

			string filename = uri.Segments[uri.Segments.Length - 1];

			string possiblePostName = filename.Substring(0, filename.Length - 5);
			if(possiblePostName.Equals(GeneratePostId(rssElement)))
				return "";
			else
				return possiblePostName;

		}

		#endregion
	}
}
