using System;
using System.Collections.Generic;
using System.Text;
using System.Collections.Specialized;
using BlogML.Xml;

namespace Rss2BlogML
{
	public sealed class RssMapperFactory
	{
		private readonly static RssMapperFactory _instance = new RssMapperFactory();

		private StringDictionary _generatorList;

		public static RssMapperFactory Current
		{
			get
			{
				return _instance;
			}
		}

		private RssMapperFactory()
		{
			//In the future this will read the configuration file
			_generatorList = new StringDictionary();
			_generatorList.Add("dotText", ".Text Version 0.95");
			_generatorList.Add("DNN", "DotNetNuke Blog 3.2.0");
		}

		public StringDictionary GeneratorList
		{
			get
			{
				return _generatorList;
			}
		}

		public IRssMapper Create(string key, string category, BlogMLBlog blog)
		{
			switch (key.ToLower())
			{
				case "dottext":
					return new DotTextRssMapper(category, blog);
				case "dnn":
					return new DnnRssMapper(category, blog);
				default:
					throw new ArgumentException(key + " is not a valid RssMapper");
			}
		}

	}
}
