using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml.Schema;
using System.Xml;

namespace BlogML.XmlSchemaValidator {
    public partial class Form1 : Form {

        List<string> validationErrors = new List<string>();
        XmlReader reader = null;


        public Form1() {
            InitializeComponent();
        }


        protected override void OnLoad(EventArgs e) {
            base.OnLoad(e);

            string schemaText = "";
            string blogmlText = "";

            this.txtSchema.Text = schemaText;
            this.txtXml.Text = blogmlText;
        }


        private void btnLoadSchema_Click(object sender, EventArgs e) {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                string text = File.ReadAllText(openFileDialog1.FileName);
                this.txtSchema.Text = text;
            }
        }

        private void btnLoadXML_Click(object sender, EventArgs e) {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                string text = File.ReadAllText(openFileDialog1.FileName);
                this.txtXml.Text = text;
            }
        }

        private void btnValidate_Click(object sender, EventArgs e) {

            try {
                ValidateXml(this.txtXml.Text, this.txtSchema.Text);
                if (validationErrors.Count > 0) {

                    StringBuilder sb = new StringBuilder();
                    foreach (string error in validationErrors) {
                        sb.AppendFormat("{0}{1}", error, Environment.NewLine);
                    }

                    MessageBox.Show(sb.ToString(), "Validation Errors", MessageBoxButtons.OKCancel, MessageBoxIcon.Error);
                } else {
                    MessageBox.Show("XML is Valid!", "XML Validation", MessageBoxButtons.OKCancel, MessageBoxIcon.Information);
                }
            } catch (XmlException ex) {
                MessageBox.Show(ex.ToString());
            }
        }



        public List<string> ValidateXml(string xml, string schemaXml) {
        
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            return ReadSchema(doc, GetSchema(schemaXml));
       
        }



        private List<string> ReadSchema(XmlDocument doc, XmlSchema schema) {
            validationErrors = new List<string>();
            try {
                XmlSchemaSet schemaSet = new XmlSchemaSet();
                schemaSet.Add(schema);

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.ValidationType = ValidationType.Schema;
                settings.ValidationFlags = XmlSchemaValidationFlags.ReportValidationWarnings;
                settings.CloseInput = true;
                settings.ConformanceLevel = ConformanceLevel.Auto;
                settings.Schemas = schemaSet;
                settings.ValidationEventHandler += new ValidationEventHandler(ValidationCallBack);

                // Parse the file. 
                XmlNodeReader nodeReader = new XmlNodeReader(doc);
                reader = XmlReader.Create(nodeReader, settings);
                while (reader.Read()) { }
            } catch (XmlException ex) {
                validationErrors.Add(String.Format("Validation Error: The Xml is invalid or not well formed - {0},{3} Line: {1},{3} Position: {2}{3}", ex.Message, ex.LineNumber.ToString(), ex.LinePosition.ToString(), Environment.NewLine));
            }

            return validationErrors;
        }


        private XmlSchema GetSchema(string text) {

            TextReader tr = new StringReader(text);
            XmlSchema schema = XmlSchema.Read(tr, new ValidationEventHandler(ValidationCallBack));
            return schema;

        }


        private void ValidationCallBack(object sender, ValidationEventArgs e) {
            validationErrors.Add(String.Format("Schema Validation Error: {0},{3} {1}{3},node name: {2}{3}", e.Message, e.Severity, reader.Name, Environment.NewLine));
        }



        private void txtSchema_TextChanged(object sender, EventArgs e) {
            bool isEnabled = (this.txtSchema.Text.Length > 0 && this.txtXml.Text.Length > 0);
            this.btnValidate.Enabled = isEnabled;
        }

        private void txtXml_TextChanged(object sender, EventArgs e) {
            bool isEnabled = (this.txtSchema.Text.Length > 0 && this.txtXml.Text.Length > 0);
            this.btnValidate.Enabled = isEnabled;
        }
    }
}