using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;
using System.Web;
using MarkItUp.SingleUserBlog.Common;
using MarkItUp.SingleUserBlog.Services;
using BlogML;
using BlogML.Xml;
using System.Drawing;
using System.Drawing.Imaging;

namespace MarkItUp.SingleUserBlog.UI {

    public sealed class SUBBlogMLReader {

        public void LoadBlog(string xml) {

            BlogMLBlog blog = BlogMLSerializer.Deserialize(new StringReader(xml));

            // mapping between categories ID and actual ids
            Hashtable categories = new System.Collections.Hashtable();

            // add categories
            foreach (BlogMLCategory category in blog.Categories) {
                Services.Category cat = new Services.Category();
                cat.Id = category.ID;
                cat.DateCreated = category.DateCreated;
                cat.DateModified = category.DateModified;
                cat.Description = category.Description;
                cat.DisplayName = category.Title;
                cat.IsApproved = category.Approved;
                if (category.ParentRef != null)
                    cat.ParentId = category.ParentRef;
                cat.CategoryType = CategoryType.Content;

                cat = DataLayer.Instance.SaveCategory(cat);
                if (category.ID != null && category.ID.Length > 0)
                    categories.Add(category.ID, cat.Id);
            }

            // add posts
            foreach (BlogMLPost post in blog.Posts) {

                string postContent = post.Content.UncodedText;

                if (post.Attachments.Count > 0) {

                    string dirPath = HttpContext.Current.Server.MapPath("~/Data/Images");
                    if (!Directory.Exists(dirPath))
                        Directory.CreateDirectory(dirPath);

                    for (int i = 0; i < post.Attachments.Count; ++i) {
                        BlogMLAttachment attachment = post.Attachments[i];

                        string fileName = Path.GetFileName(attachment.Url);
                        string filePath = HttpContext.Current.Server.MapPath("~/Data/Images/" + fileName);
                        string httpPath = Globals.UrlStartPath + "Data/Images/" + fileName;

                        postContent = BlogMLWriterBase.SgmlUtil.CleanAttachmentUrls(postContent, attachment.Url, httpPath);

                        if (attachment.Embedded == true && !File.Exists(filePath)) {
                            MemoryStream ms = new MemoryStream(attachment.Data);
                            Bitmap b = (Bitmap)Image.FromStream(ms);
                            using (FileStream fs = new FileStream(filePath, FileMode.CreateNew)) {
                                b.Save(fs, ImageFormat.Jpeg);
                            }
                        }
                    }
                }

                Services.Post p = new Services.Post();
                p.Id = post.ID;
                p.DateCreated = post.DateCreated;
                p.DateModified = post.DateModified;
                p.DisplayName = post.Title;
                p.IsApproved = post.Approved;
                p.Description = postContent;
                p = DataLayer.Instance.SavePost(p);

                // attaching categories
                if (post.Categories.Count > 0) {
                    string[] catIds = new string[post.Categories.Count];
                    for (int i = 0; i < post.Categories.Count; ++i) {
                        catIds[i] = (string)categories[post.Categories[i].Ref];
                    }
                    DataLayer.Instance.SavePostCategories(p.Id, catIds);
                }

                // attaching comments
                if (post.Comments.Count > 0) {
                    foreach (BlogMLComment comment in post.Comments) {
                        FeedbackItem item = new FeedbackItem();
                        item.Id = comment.ID;
                        item.DateCreated = comment.DateCreated;
                        item.DateModified = comment.DateModified;
                        item.Description = comment.Content.UncodedText;
                        item.DisplayName = comment.Title;
                        item.IsApproved = comment.Approved;
                        item.PostId = post.ID;
                        item.Username = comment.UserName;
                        item.UserUrl = comment.UserUrl;

                        DataLayer.Instance.SaveFeedback(item);
                    }
                }
            }
        }
    }
}
