using System;
using System.Collections;
using System.Collections.Specialized;
using System.Xml.Serialization;
using System.Collections.Generic;
using System.Reflection;

namespace BlogML.Xml
{
    [Serializable]
    [XmlRoot(ElementName = "blog", Namespace = "http://www.blogml.com/2006/09/BlogML")]
    public sealed class BlogMLBlog
    {
        private string rootUrl;
        private string title;
        private string subTitle;
        private DateTime dateCreated = DateTime.Now;
        private AuthorCollection authors = new AuthorCollection();
        private PostCollection posts = new PostCollection();
        private CategoryCollection categories = new CategoryCollection();
        private ExtendedPropertiesCollection extendedproperties = new ExtendedPropertiesCollection();


        [XmlAttribute("root-url")]
        public string RootUrl
        {
            get { return this.rootUrl; }
            set { this.rootUrl = value; }
        }

        [XmlElement("title")]
        public string Title
        {
            get { return this.title; }
            set { this.title = value; }
        }

        [XmlElement("sub-title")]
        public string SubTitle
        {
            get { return this.subTitle; }
            set { this.subTitle = value; }
        }

        [XmlAttribute("date-created", DataType = "dateTime")]
        public DateTime DateCreated
        {
            get { return this.dateCreated; }
            set { this.dateCreated = value; }
        }

        [XmlArray("extended-properties")]
        [XmlArrayItem("property", typeof(Pair<string, string>))]
        public ExtendedPropertiesCollection ExtendedProperties
        {
            get
            {
                return this.extendedproperties;
            }
        }

        [XmlArray("authors")]
        [XmlArrayItem("author", typeof(BlogMLAuthor))]
        public AuthorCollection Authors
        {
            get
            {
                return this.authors;
            }
        }


        [XmlArray("posts")]
        [XmlArrayItem("post", typeof(BlogMLPost))]
        public PostCollection Posts
        {
            get
            {
                return this.posts;
            }
        }

        [XmlArray("categories")]
        [XmlArrayItem("category", typeof(BlogMLCategory))]
        public CategoryCollection Categories
        {
            get
            {
                return this.categories;
            }
        }

        [Serializable]
        public sealed class AuthorCollection : List<BlogMLAuthor>
        {
            public override bool Equals(object obj)
            {
                AuthorCollection authors = obj as AuthorCollection;

                int i = 0;

                foreach (BlogMLAuthor author in authors)
                {
                    BlogMLAuthor original = this[i];

                    if ((author.ID != original.ID) ||
                        (author.Title != original.Title) ||
                        (author.Email != original.Email) ||
                        (!author.DateCreated.Equals(original.DateCreated)) ||
                        (!author.DateModified.Equals(original.DateModified)) ||
                        (author.Approved != original.Approved))
                        return false;
                    i++;
                }
                return true;
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }

        [Serializable]
        public sealed class PostCollection : List<BlogMLPost>
        {
            public override bool Equals(object obj)
            {
                PostCollection posts = obj as PostCollection;

                int i = 0;
                foreach (BlogMLPost post in posts)
                {
                    BlogMLPost original = this[i];

                    foreach (PropertyInfo property in
                        Type.GetType(typeof(BlogMLPost).ToString()).GetProperties())
                    {
                        // Read the blog post here:
                        // http://nayyeri.net/blog/when-false-false-is-evaluated-as-true/
                        object postPropertyValue = property.GetValue(post, null);
                        object originalPropertyValue = property.GetValue(original, null);

                        if (postPropertyValue != null)
                        {
                            if (postPropertyValue.Equals(originalPropertyValue) == false)
                                return false;
                        }
                        else if (postPropertyValue == null && originalPropertyValue != null)
                        {
                            return false;
                        }
                    }
                    i++;
                }

                return true;
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }

        [Serializable]
        public sealed class CategoryCollection : List<BlogMLCategory>
        {
            public override bool Equals(object obj)
            {
                CategoryCollection categories = obj as CategoryCollection;

                int i = 0;

                foreach (BlogMLCategory category in categories)
                {
                    BlogMLCategory original = this[i];

                    if ((category.ID != original.ID) ||
                        (category.Title != original.Title) ||
                        (!category.DateCreated.Equals(original.DateCreated)) ||
                        (!category.DateModified.Equals(original.DateModified)) ||
                        (category.Approved != original.Approved) ||
                        (category.ParentRef != original.ParentRef) ||
                        (category.Description != original.Description))
                        return false;
                    i++;
                }

                return true;
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }

        [Serializable]
        public sealed class ExtendedPropertiesCollection : List<Pair<string, string>>
        {
            public override bool Equals(object obj)
            {
                ExtendedPropertiesCollection collection = obj as ExtendedPropertiesCollection;

                int i = 0;
                foreach (Pair<string, string> pair in this)
                {
                    if ((pair.Key != collection[i].Key) || (pair.Value != collection[i].Value))
                        return false;
                    i++;
                }
                return true;
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }
    }
}
