﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.IO;

namespace WpfSchemaValidator
{
    internal class Validator
    {
        private List<ValidationResult> validationErrors =
            new List<ValidationResult>();
        private XmlReader reader;

        public List<ValidationResult> ValidateXml(string xml, string schema)
        {
            XmlDocument xmlDoc = new XmlDocument();
            XmlSchema xmlSchema = new XmlSchema();
            try
            {
                xmlDoc.LoadXml(xml);

                xmlSchema = GetSchema(schema);
            }
            catch (XmlException xmlEx)
            {
                AddXmlExceptions(xmlEx);

                return this.validationErrors;
            }

            ParseAndValidate(xmlDoc, xmlSchema);

            return this.validationErrors;
        }

        private XmlSchema GetSchema(string text)
        {
            TextReader textReader = new StringReader(text);
            XmlSchema schema = XmlSchema.Read(textReader,
                new ValidationEventHandler(ValidationCallBack));
            return schema;
        }

        private void ParseAndValidate(XmlDocument xml, XmlSchema schema)
        {
            try
            {
                XmlSchemaSet schemaSet = new XmlSchemaSet();
                schemaSet.Add(schema);

                XmlReaderSettings settings = new XmlReaderSettings();
                settings.ValidationType = ValidationType.Schema;
                settings.ValidationFlags =
                    XmlSchemaValidationFlags.ReportValidationWarnings;
                settings.CloseInput = true;
                settings.ConformanceLevel = ConformanceLevel.Auto;
                settings.Schemas = schemaSet;
                settings.ValidationEventHandler +=
                    new ValidationEventHandler(ValidationCallBack);

                XmlNodeReader nodeReader = new XmlNodeReader(xml);
                reader = XmlReader.Create(nodeReader, settings);

                while (reader.Read()) { }
            }

            catch (UnauthorizedAccessException accessEx)
            {
                throw accessEx;
            }
            catch (XmlException xmlEx)
            {
                AddXmlExceptions(xmlEx);
            }
        }

        private void AddXmlExceptions(XmlException xmlEx)
        {
            ValidationResult result = new ValidationResult();
            result.Type = XmlSeverityType.Error;
            result.Error = xmlEx.Message;
            result.Row = xmlEx.LineNumber;
            result.Column = xmlEx.LinePosition;

            this.validationErrors.Add(result);
        }

        private void ValidationCallBack(object sender, ValidationEventArgs e)
        {
            ValidationResult result = new ValidationResult();
            result.Type = e.Severity;
            result.Error = e.Message;
            result.Row = e.Exception.LineNumber;
            result.Column = e.Exception.LinePosition;

            this.validationErrors.Add(result);
        }
    }
}
