﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Forms;
using System.IO;
using Xceed.Wpf.DataGrid;
using System.Xml.Schema;
using System.Resources;
using System.Reflection;

namespace WpfSchemaValidator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private int originalHeight = 465;
        private int extendedHeight = 600;
        private int gridHeight = 150;

        public MainWindow()
        {
            InitializeComponent();


            // Load and add columns
            Column type = new Column();
            type.Title = "Type";
            type.FieldName = "Type";
            type.Width = 50;            

            Column error = new Column();
            error.Title = "Error";
            error.FieldName = "Error";
            error.Width = 400;
            error.TextWrapping = TextWrapping.Wrap;

            Column row = new Column();
            row.Title = "Row";
            row.FieldName = "Row";
            row.Width = 50;

            Column column = new Column();
            column.Title = "Column";
            column.FieldName = "Column";
            column.Width = 50;

            resultsGrid.Columns.Add(type);
            resultsGrid.Columns.Add(error);
            resultsGrid.Columns.Add(row);
            resultsGrid.Columns.Add(column);

            // Icon
            MemoryStream iconStream = new MemoryStream();
            WpfSchemaValidator.Properties.Resources.Icon.Save(iconStream);
            this.Icon = BitmapFrame.Create(iconStream);

            // Load BlogML schema
            this.txtXsd.Text = WpfSchemaValidator.Properties.Resources.BlogML;
        }

        private void btnValidate_Click(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(txtXml.Text) || string.IsNullOrEmpty(txtXsd.Text))
            {
                System.Windows.Forms.MessageBox.Show("XML and/or Schema text not available!",
                    "XML and Schema Content");
                return;
            }

            try
            {
                resultsGrid.Items.Clear();

                Validator validator = new Validator();

                List<ValidationResult> validationResults =
                    validator.ValidateXml(this.txtXml.Text, this.txtXsd.Text);

                if (validationResults.Count > 0)
                {
                    foreach (ValidationResult validationResult in validationResults)
                        resultsGrid.Items.Add(validationResult);

                    this.Height = this.extendedHeight;
                    resultsGrid.Visibility = Visibility.Visible;
                    resultsGrid.Height = this.gridHeight;

                    if (validationResults.Exists(IsError))
                    {
                        btnValidate.Background = Brushes.Red;
                        btnValidate.Content = "Invalid";
                    }
                    else
                    {
                        btnValidate.Background = Brushes.Orange;
                        btnValidate.Content = "Valid";
                    }
                }
                else
                {
                    btnValidate.Background = Brushes.Green;
                    btnValidate.Content = "Valid";
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.ToString(), "Exception");
            }
        }

        private bool IsError(ValidationResult item)
        {
            return ((item.Type == XmlSeverityType.Error) ? true : false);
        }


        private void xmlSlider_ValueChanged(object sender,
            RoutedPropertyChangedEventArgs<double> e)
        {
            double value = e.NewValue;

            this.txtXml.FontSize = value * 5;
        }

        private void xsdSlider_ValueChanged(object sender,
            RoutedPropertyChangedEventArgs<double> e)
        {
            double value = e.NewValue;

            this.txtXsd.FontSize = value * 5;
        }

        private void txtXml_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "XML Files (*.xml)|*.xml|All Files (*.*)|*.*";
            dialog.ShowDialog();
            if (!string.IsNullOrEmpty(dialog.FileName))
                txtXml.Text = File.ReadAllText(dialog.FileName);
        }

        private void txtXsd_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.Filter = "XSD Files (*.xsd)|*.xsd|All Files (*.*)|*.*";
            dialog.ShowDialog();
            if (!string.IsNullOrEmpty(dialog.FileName))
                txtXsd.Text = File.ReadAllText(dialog.FileName);
        }

        private void txt_TextChanged(object sender, TextChangedEventArgs e)
        {
            this.Height = this.originalHeight;
            resultsGrid.Visibility = Visibility.Hidden;
            btnValidate.Background = Brushes.Black;
            btnValidate.Content = "Validate";
        }
    }
}
